import os, re, time, urllib
from Cheetah.Template import Template


url_re = r"(((https?|ftp|gopher|telnet)://|(mailto|file|news|about|ed2k|irc|sip|magnet):)[^' \t<>\"]+|(www|web|w3)[A-Za-z0-9_-]*\.[A-Za-z0-9._-]+\.[^' \t<>\"]+)[A-Za-z0-9/]"

def _url2href(match):
   url = match.group(0)
   return '<a href="%s">%s</a>' % (url, url)


full_dirs = len(os.getcwd().split('/')) + 1

class phd(Template):
   def __init__(self, *args, **kw):
      if not hasattr(self, "_fileBaseName"):
         self._fileDirName, self._fileBaseName = os.path.split(os.path.abspath(self._CHEETAH_src))
      Template.__init__(self, *args, **kw)
      directories = self._fileDirName.split('/')[full_dirs:] # remove directories up to "./files"
      dirs_to_root = len(directories)
      if dirs_to_root:
         root = "../"*dirs_to_root
      else:
         root = ''
      self.root = root
      path = '/'.join(directories) + '/' + \
         self._fileBaseName.replace(".tmpl", ".html")
      if path[0] <> '/': path = '/' + path
      self.path = path

   def copyright(self, start_year):
      this_year = time.localtime()[0]
      if start_year >= this_year:
         return this_year
      if start_year == this_year - 1:
         return "%s, %s" % (start_year, this_year)
      return "%s-%s" % (start_year, this_year)


   def body(self):
      if hasattr(self, "body_html"):
         return self.body_html()
      if hasattr(self, "body_text"):
         return self.text2html()
      if hasattr(self, "body_rst"):
         return self.rst2html()
      if hasattr(self, "body_mkd"):
         return self.mkd2html()

   def text2html(self):
      body = re.sub(url_re, _url2href, self.body_text())

      paragraphs = body.split("\n\n")

      new_paras = []
      for p in paragraphs:
         if isinstance(p, unicode):
            p = p.encode('utf-8')
         parts = p.split("\n   ")
         parts[0] = parts[0].strip()
         new_paras.append('\n</p>\n<p>\n'.join(parts))

      if self.Title:
         title = "<h1>%s</h1>\n\n" % self.Title
      else:
         title = ''

      body = '\n</p>\n\n<p class="head">\n'.join(new_paras)
      return "%s<p>%s</p>" % (title, body)

   def rst2html(self):
      from docutils.core import publish_parts

      parts = publish_parts(self.body_rst(), writer_name="html")

      title = parts["title"] or self.Title
      if title:
         title = "<h1>%s</h1>" % title

      subtitle = parts["subtitle"]
      if subtitle:
         subtitle = "<h2>%s</h2>" % subtitle

      body = parts["body"]
      parts = []
      for part in (title, subtitle, body):
          if not part:
              continue
          if isinstance(part, unicode):
              part = part.encode('utf-8')
          parts.append(part)
      return "\n\n".join(parts)

   def mkd2html(self):
      from markdown import markdown
      return markdown(self.body_mkd(), output_format="html")

   def img_thumbnail_800_1024(self, img_name):
      return """\
<img src="%(img_name)s-thumbnail.jpg" alt="%(img_name)s-thumbnail.jpg" /><br />
<a href="%(img_name)s-800x600.jpg">800x600</a>, <a href="%(img_name)s-1024x800.jpg">1024x800</a>""" % {"img_name": img_name}

   def wikipedia(self, query):
      return "http://en.wikipedia.org/wiki/%s" % quote_string(query.replace(' ', '_'), ext_safe=',')

   def wikipedia_ru(self, query):
      return "http://ru.wikipedia.org/wiki/%s" % quote_string(query.replace(' ', '_'), ext_safe=',')

   def nigma(self, query):
       return "http://www.nigma.ru/index.php?s=%s" % quote_string(query)

   search = nigma

   def yandex(self, query):
      return "http://www.yandex.ru/yandsearch?text=%s&rpt=rad" % quote_string(query, "cp1251")

   def google(self, query):
      return "http://www.google.com/search?hl=en&ie=utf-8&oe=utf-8&q=%s" % quote_string(query)

   def google_ru(self, query):
      return "http://www.google.ru/search?hl=ru&ie=utf-8&oe=utf-8&q=%s" % quote_string(query)

def quote_string(s, to_encoding="utf-8", ext_safe=''):
   return urllib.quote(unicode(s, "utf-8").encode(to_encoding), '/' + ext_safe)
